#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMetaProperty>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi (this);
    displayObject (ui->pushButton);
}

MainWindow::~MainWindow()
{
    delete ui;
}

QTableWidgetItem * MainWindow::displayLine (QString name, QString value)
{
    int line = line_count;
    line_count ++;
    if (ui->tableWidget->rowCount() < line_count)
        ui->tableWidget->setRowCount (line_count);

    QTableWidgetItem * item = new QTableWidgetItem;
    item->setText (name);
    item->setForeground(QColor ("blue"));
    ui->tableWidget->setItem (line, 0, item);

    item = new QTableWidgetItem;
    item->setText (value);
    item->setForeground(QColor ("blue"));
    ui->tableWidget->setItem (line, 1, item);

    return item;
}

void MainWindow::displayProperty (const QObject * obj, QMetaProperty prop)
{
    QString name = prop.name();
    QVariant value = prop.read (obj);
    QTableWidgetItem * item = displayLine (name, value.toString());

    QVariant::Type t = value.type();
    if (t == QVariant::Bool)
    {
       bool b = value.toBool ();
       item->setData (Qt::CheckStateRole, b ? Qt::Checked : Qt::Unchecked);
    }
    if (t == QVariant::Color)
    {
        QColor c = value.value <QColor> ();
        item->setData (Qt::DecorationRole, c);
    }
    if (t == QVariant::Palette)
    {
        QPalette p = value.value <QPalette> ();
        QColor c = p.background().color();
        item->setData (Qt::DecorationRole, c);
    }
    if (t == QVariant::Icon)
    {
        QIcon i = value.value <QIcon> ();
        item->setData (Qt::DecorationRole, i);
    }
    if (t == QVariant::Image)
    {
        QImage i = value.value <QImage> ();
        item->setData (Qt::DecorationRole, i);
    }
    // if (t == QVariant::Color)
}

void MainWindow::displayObject (QObject * obj)
{
    line_count = 0;
    ui->tableWidget->setColumnCount(2);
    ui->tableWidget->setRowCount(10);

    QStringList list;
    list << "Name" << "Value";
    ui->tableWidget->setHorizontalHeaderLabels (list);

    const QMetaObject * type = obj->metaObject ();
    displayLine("className", type->className());

    int cnt = type->propertyCount();
    for (int inx = 0; inx < cnt; inx++)
    {
        // #include <QMetaProperty>
        QMetaProperty prop = type->property (inx);
        displayProperty (obj, prop);
    }
}
